@@ -7,7 +7,8 @@ the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "http" or "web"
 or "html" (this is the default).
 
-Copyright 2004-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
+               2008 H. Turgut Uyar <uyar@tekir.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -28,32 +29,53 @@ import sys
 from urllib import FancyURLopener, quote_plus
 from codecs import lookup
 
-from imdb import IMDbBase, imdbURL_movie_main, imdbURL_person_main, imdbURL_find
-from imdb.Movie import Movie
-from imdb.utils import analyze_title
+from imdb import IMDbBase, imdbURL_movie_main, imdbURL_person_main, \
+                imdbURL_character_main, imdbURL_company_main, imdbURL_find
 from imdb._exceptions import IMDbDataAccessError, IMDbParserError
-from movieParser import movie_parser, plot_parser, movie_awards_parser, \
-                        taglines_parser, keywords_parser, \
-                        alternateversions_parser, crazycredits_parser, \
-                        goofs_parser, trivia_parser, quotes_parser, \
-                        releasedates_parser, ratings_parser, \
-                        officialsites_parser, connections_parser, \
-                        tech_parser, locations_parser, soundtrack_parser, \
-                        dvd_parser, rec_parser, externalrev_parser, \
-                        newsgrouprev_parser, misclinks_parser, \
-                        soundclips_parser, videoclips_parser, news_parser, \
-                        photosites_parser, amazonrev_parser, business_parser, \
-                        literature_parser, sales_parser, episodes_parser, \
-                        eprating_parser, movie_faqs_parser, airing_parser, \
-                        synopsis_parser, parentsguide_parser
-from searchMovieParser import search_movie_parser
-from personParser import maindetails_parser, bio_parser, \
-                        otherworks_parser, person_awards_parser, \
-                        person_officialsites_parser, publicity_parser, \
-                        agent_parser, person_series_parser, \
-                        person_genres_parser, person_keywords_parser
-from searchPersonParser import search_person_parser
-from utils import ParserBase
+
+import searchMovieParser
+import searchPersonParser
+import searchCharacterParser
+import searchCompanyParser
+import movieParser
+import personParser
+import characterParser
+import companyParser
+
+
+class _ModuleProxy:
+    """A proxy to instantiate and access parsers."""
+    def __init__(self, module, defaultKeys=None, oldParsers=False,
+                useModule=None):
+        """Initialize a proxy for the given module; defaultKeys, if set,
+        muste be a dictionary of values to set for instanced objects."""
+        self.oldParsers = oldParsers
+        self.useModule = useModule
+        if defaultKeys is None:
+            defaultKeys = {}
+        self._defaultKeys = defaultKeys
+        self._module = module
+
+    def __getattr__(self, name):
+        """Called only when no look-up is found."""
+        _sm = self._module
+        # Read the _OBJECTS dictionary to build the asked parser.
+        if name in _sm._OBJECTS:
+            _entry = _sm._OBJECTS[name]
+            # Initialize the parser.
+            kwds = {}
+            if not self.oldParsers and self.useModule:
+                kwds = {'useModule': self.useModule}
+            obj = _entry[0][self.oldParsers](**kwds)
+            attrsToSet = self._defaultKeys.copy()
+            attrsToSet.update(_entry[1] or {})
+            # Set attribute to the object.
+            for key in attrsToSet:
+                setattr(obj, key, attrsToSet[key])
+            setattr(self, name, obj)
+            return obj
+        return getattr(_sm, name)
+
 
 PY_VERSION = sys.version_info[:2]
 
@@ -71,6 +93,7 @@ _cookie_uu = 'su4/m8cho4c6HP+W1qgq6wchOmhnF0w+lIWvHjRUPJ6nRA9sccEafjGADJ6hQGrMd4
 class IMDbURLopener(FancyURLopener):
     """Fetch web pages and handle errors."""
     def __init__(self, *args, **kwargs):
+        self._last_url = u''
         FancyURLopener.__init__(self, *args, **kwargs)
         # Headers to add to every request.
         # XXX: IMDb's web server doesn't like urllib-based programs,
@@ -122,6 +145,7 @@ class IMDbURLopener(FancyURLopener):
             if PY_VERSION > (2, 3):
                 kwds['size'] = size
             content = uopener.read(**kwds)
+            self._last_url = uopener.url
             # Maybe the server is so nice to tell us the charset...
             server_encode = uopener.info().getparam('charset')
             # Otherwise, look at the content-type HTML meta tag.
@@ -186,7 +210,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     accessSystem = 'http'
 
-    def __init__(self, isThin=0, adultSearch=1, proxy=-1,
+    def __init__(self, isThin=0, adultSearch=1, proxy=-1, oldParsers=False,
+                useModule=None, cookie_id=-1, cookie_uu=None,
                 *arguments, **keywords):
         """Initialize the access system."""
         IMDbBase.__init__(self, *arguments, **keywords)
@@ -198,15 +223,39 @@ class IMDbHTTPAccessSystem(IMDbBase):
         self._getRefs = True
         self._mdparse = False
         if isThin:
-            self.accessSystem = 'httpThin'
+            if self.accessSystem == 'http':
+                self.accessSystem = 'httpThin'
             self._mdparse = True
             if self._defModFunct is None:
                 self._getRefs = False
                 from imdb.utils import modNull
                 self._defModFunct = modNull
         self.do_adult_search(adultSearch)
+        if cookie_id != -1:
+            if cookie_id is None:
+                self.del_cookies()
+            elif cookie_uu is not None:
+                self.set_cookies(cookie_id, cookie_uu)
         if proxy != -1:
             self.set_proxy(proxy)
+        _def = {'_modFunct': self._defModFunct, '_as': self.accessSystem}
+        # Proxy objects.
+        self.smProxy = _ModuleProxy(searchMovieParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule)
+        self.spProxy = _ModuleProxy(searchPersonParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule)
+        self.scProxy = _ModuleProxy(searchCharacterParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule)
+        self.scompProxy = _ModuleProxy(searchCompanyParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule)
+        self.mProxy = _ModuleProxy(movieParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule)
+        self.pProxy = _ModuleProxy(personParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule)
+        self.cProxy = _ModuleProxy(characterParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule)
+        self.compProxy = _ModuleProxy(companyParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule)
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -222,6 +271,22 @@ class IMDbHTTPAccessSystem(IMDbBase):
         except ValueError, e:
             raise IMDbParserError, 'invalid personID "%s": %s' % (personID, e)
 
+    def _normalize_characterID(self, characterID):
+        """Normalize the given characterID."""
+        try:
+            return '%07d' % int(characterID)
+        except ValueError, e:
+            raise IMDbParserError, 'invalid characterID "%s": %s' % \
+                    (characterID, e)
+
+    def _normalize_companyID(self, companyID):
+        """Normalize the given companyID."""
+        try:
+            return '%07d' % int(companyID)
+        except ValueError, e:
+            raise IMDbParserError, 'invalid companyID "%s": %s' % \
+                    (companyID, e)
+
     def get_imdbMovieID(self, movieID):
         """Translate a movieID in an imdbID; in this implementation
         the movieID _is_ the imdbID.
@@ -234,6 +299,18 @@ class IMDbHTTPAccessSystem(IMDbBase):
         """
         return personID
 
+    def get_imdbCharacterID(self, characterID):
+        """Translate a characterID in an imdbID; in this implementation
+        the characterID _is_ the imdbID.
+        """
+        return characterID
+
+    def get_imdbCompanyID(self, companyID):
+        """Translate a companyID in an imdbID; in this implementation
+        the companyID _is_ the imdbID.
+        """
+        return companyID
+
     def get_proxy(self):
         """Return the used proxy or an empty string."""
         return self.urlOpener.get_proxy()
@@ -248,6 +325,15 @@ class IMDbHTTPAccessSystem(IMDbBase):
         """
         self.urlOpener.set_proxy(proxy)
 
+    def set_cookies(self, cookie_id, cookie_uu):
+        """Set a cookie to access an IMDb's account."""
+        c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
+        self.urlOpener.set_header('Cookie', c_header)
+
+    def del_cookies(self):
+        """Remove the used cookie."""
+        self.urlOpener.del_header('Cookie')
+
     def do_adult_search(self, doAdult,
                         cookie_id=_cookie_id, cookie_uu=_cookie_uu):
         """If doAdult is true, 'adult' movies are included in the
@@ -255,18 +341,21 @@ class IMDbHTTPAccessSystem(IMDbBase):
         parameters to select a specific account (see your cookie
         or cookies.txt file."""
         if doAdult:
-            c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
-            self.urlOpener.set_header('Cookie', c_header)
+            self.set_cookies(cookie_id, cookie_uu)
+            #c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
+            #self.urlOpener.set_header('Cookie', c_header)
         else:
             self.urlOpener.del_header('Cookie')
 
     def _retrieve(self, url, size=-1):
         """Retrieve the given URL."""
+        ##print url
         return self.urlOpener.retrieve_unicode(url, size=size)
 
     def _get_search_content(self, kind, ton, results):
         """Retrieve the web page for a given search.
-        kind can be tt (for titles) or nm (for names)
+        kind can be 'tt' (for titles), 'nm' (for names),
+        'char' (for characters) or 'co' (for companies).
         ton is the title or the name to search.
         results is the maximum number of results to be retrieved."""
         if isinstance(ton, unicode):
@@ -274,6 +363,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
         ##params = 'q=%s&%s=on&mx=%s' % (quote_plus(ton), kind, str(results))
         params = 's=%s;mx=%s;q=%s' % (kind, str(results), quote_plus(ton))
         cont = self._retrieve(imdbURL_find % params)
+        #print 'URL:', imdbURL_find % params
         if cont.find('more than 500 partial matches') == -1:
             return cont
         # The retrieved page contains no results, because too many
@@ -290,143 +380,145 @@ class IMDbHTTPAccessSystem(IMDbBase):
         ##params = 'q=%s&tt=on&mx=%s' % (quote_plus(title), str(results))
         ##cont = self._retrieve(imdbURL_find % params)
         cont = self._get_search_content('tt', title, results)
-        return search_movie_parser.parse(cont, results=results)['data']
+        return self.smProxy.search_movie_parser.parse(cont, results=results)['data']
 
     def get_movie_main(self, movieID):
         if not self.isThin:
             cont = self._retrieve(imdbURL_movie_main % movieID + 'combined')
         else:
             cont = self._retrieve(imdbURL_movie_main % movieID + 'maindetails')
-        return movie_parser.parse(cont, mdparse=self._mdparse)
+        return self.mProxy.movie_parser.parse(cont, mdparse=self._mdparse)
 
     def get_movie_full_credits(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'fullcredits')
-        return movie_parser.parse(cont)
+        return self.mProxy.movie_parser.parse(cont)
 
     def get_movie_plot(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'plotsummary')
-        return plot_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.plot_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_awards(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'awards')
-        return movie_awards_parser.parse(cont)
+        return self.mProxy.movie_awards_parser.parse(cont)
 
     def get_movie_taglines(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'taglines')
-        return taglines_parser.parse(cont)
+        return self.mProxy.taglines_parser.parse(cont)
 
     def get_movie_keywords(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'keywords')
-        return keywords_parser.parse(cont)
+        return self.mProxy.keywords_parser.parse(cont)
 
     def get_movie_alternate_versions(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'alternateversions')
-        return alternateversions_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.alternateversions_parser.parse(cont,
+                                                        getRefs=self._getRefs)
 
     def get_movie_crazy_credits(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'crazycredits')
-        return crazycredits_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.crazycredits_parser.parse(cont,
+                                                    getRefs=self._getRefs)
 
     def get_movie_goofs(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'goofs')
-        return goofs_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.goofs_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_quotes(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'quotes')
-        return quotes_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.quotes_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_release_dates(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'releaseinfo')
-        return releasedates_parser.parse(cont)
+        return self.mProxy.releasedates_parser.parse(cont)
 
     def get_movie_vote_details(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'ratings')
-        return ratings_parser.parse(cont)
+        return self.mProxy.ratings_parser.parse(cont)
 
     def get_movie_official_sites(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'officialsites')
-        return officialsites_parser.parse(cont)
+        return self.mProxy.officialsites_parser.parse(cont)
 
     def get_movie_trivia(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'trivia')
-        return trivia_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.trivia_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_connections(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'movieconnections')
-        return connections_parser.parse(cont)
+        return self.mProxy.connections_parser.parse(cont)
 
     def get_movie_technical(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'technical')
-        return tech_parser.parse(cont)
+        return self.mProxy.tech_parser.parse(cont)
 
     def get_movie_business(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'business')
-        return business_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.business_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_literature(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'literature')
-        return literature_parser.parse(cont)
+        return self.mProxy.literature_parser.parse(cont)
 
     def get_movie_locations(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'locations')
-        return locations_parser.parse(cont)
+        return self.mProxy.locations_parser.parse(cont)
 
     def get_movie_soundtrack(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'soundtrack')
-        return soundtrack_parser.parse(cont)
+        return self.mProxy.soundtrack_parser.parse(cont)
 
     def get_movie_dvd(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'dvd')
-        return dvd_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.dvd_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_recommendations(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'recommendations')
-        return rec_parser.parse(cont)
+        return self.mProxy.rec_parser.parse(cont)
 
     def get_movie_external_reviews(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'externalreviews')
-        return externalrev_parser.parse(cont)
+        return self.mProxy.externalrev_parser.parse(cont)
 
     def get_movie_newsgroup_reviews(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'newsgroupreviews')
-        return newsgrouprev_parser.parse(cont)
+        return self.mProxy.newsgrouprev_parser.parse(cont)
 
     def get_movie_misc_sites(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'miscsites')
-        return misclinks_parser.parse(cont)
+        return self.mProxy.misclinks_parser.parse(cont)
 
     def get_movie_sound_clips(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'soundsites')
-        return soundclips_parser.parse(cont)
+        return self.mProxy.soundclips_parser.parse(cont)
 
     def get_movie_video_clips(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'videosites')
-        return videoclips_parser.parse(cont)
+        return self.mProxy.videoclips_parser.parse(cont)
 
     def get_movie_photo_sites(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'photosites')
-        return photosites_parser.parse(cont)
+        return self.mProxy.photosites_parser.parse(cont)
 
     def get_movie_news(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'news')
-        return news_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.news_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_amazon_reviews(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'amazon')
-        return amazonrev_parser.parse(cont)
+        return self.mProxy.amazonrev_parser.parse(cont)
 
     def get_movie_guests(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'epcast')
-        return episodes_parser.parse(cont)
+        return self.mProxy.episodes_cast_parser.parse(cont)
     get_movie_episodes_cast = get_movie_guests
 
     def get_movie_merchandising_links(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'sales')
-        return sales_parser.parse(cont)
+        return self.mProxy.sales_parser.parse(cont)
 
     def get_movie_episodes(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'episodes')
-        data_d = episodes_parser.parse(cont)
+        data_d = self.mProxy.episodes_parser.parse(cont)
         # set movie['episode of'].movieID for every episode of the series.
         if data_d.get('data', {}).has_key('episodes'):
             nr_eps = 0
@@ -441,7 +533,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_episodes_rating(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'epdate')
-        data_d = eprating_parser.parse(cont)
+        data_d = self.mProxy.eprating_parser.parse(cont)
         # set movie['episode of'].movieID for every episode.
         if data_d.get('data', {}).has_key('episodes rating'):
             for item in data_d['data']['episodes rating']:
@@ -451,19 +543,19 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_faqs(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'faq')
-        return movie_faqs_parser.parse(cont, getRefs=self._getRefs)
+        return self.mProxy.movie_faqs_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_airing(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'tvschedule')
-        return airing_parser.parse(cont)
+        return self.mProxy.airing_parser.parse(cont)
 
     def get_movie_synopsis(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'synopsis')
-        return synopsis_parser.parse(cont)
+        return self.mProxy.synopsis_parser.parse(cont)
 
     def get_movie_parents_guide(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'parentalguide')
-        return parentsguide_parser.parse(cont)
+        return self.mProxy.parentsguide_parser.parse(cont)
 
     def _search_person(self, name, results):
         # The URL of the query.
@@ -473,11 +565,11 @@ class IMDbHTTPAccessSystem(IMDbBase):
         #params = 'q=%s&nm=on&mx=%s' % (quote_plus(name), str(results))
         #cont = self._retrieve(imdbURL_find % params)
         cont = self._get_search_content('nm', name, results)
-        return search_person_parser.parse(cont, results=results)['data']
+        return self.spProxy.search_person_parser.parse(cont, results=results)['data']
 
     def get_person_main(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'maindetails')
-        ret = maindetails_parser.parse(cont)
+        ret = self.pProxy.maindetails_parser.parse(cont)
         ret['info sets'] = ('main', 'filmography')
         return ret
 
@@ -486,46 +578,84 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_person_biography(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'bio')
-        return bio_parser.parse(cont, getRefs=self._getRefs)
+        return self.pProxy.bio_parser.parse(cont, getRefs=self._getRefs)
 
     def get_person_awards(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'awards')
-        return person_awards_parser.parse(cont)
+        return self.pProxy.person_awards_parser.parse(cont)
 
     def get_person_other_works(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'otherworks')
-        return otherworks_parser.parse(cont, getRefs=self._getRefs)
+        return self.pProxy.otherworks_parser.parse(cont, getRefs=self._getRefs)
 
     def get_person_agent(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'agent')
-        return agent_parser.parse(cont)
+        return self.pProxy.agent_parser.parse(cont)
 
     def get_person_publicity(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'publicity')
-        return publicity_parser.parse(cont)
+        return self.pProxy.publicity_parser.parse(cont)
 
     def get_person_official_sites(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'officialsites')
-        return person_officialsites_parser.parse(cont)
+        return self.pProxy.person_officialsites_parser.parse(cont)
 
     def get_person_news(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'news')
-        return news_parser.parse(cont)
+        return self.pProxy.news_parser.parse(cont)
 
     def get_person_episodes(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'filmoseries')
-        return person_series_parser.parse(cont)
+        return self.pProxy.person_series_parser.parse(cont)
 
     def get_person_merchandising_links(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'forsale')
-        return sales_parser.parse(cont)
+        return self.pProxy.sales_parser.parse(cont)
 
     def get_person_genres_links(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'filmogenre')
-        return person_genres_parser.parse(cont)
+        return self.pProxy.person_genres_parser.parse(cont)
 
     def get_person_keywords_links(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'filmokey')
-        return person_keywords_parser.parse(cont)
+        return self.pProxy.person_keywords_parser.parse(cont)
+
+    def _search_character(self, name, results):
+        cont = self._get_search_content('char', name, results)
+        return self.scProxy.search_character_parser.parse(cont, results=results)['data']
+
+    def get_character_main(self, characterID):
+        cont = self._retrieve(imdbURL_character_main % characterID)
+        ret = self.cProxy.character_main_parser.parse(cont)
+        ret['info sets'] = ('main', 'filmography')
+        return ret
+
+    get_character_filmography = get_character_main
+
+    def get_character_biography(self, characterID):
+        cont = self._retrieve(imdbURL_character_main % characterID + 'bio')
+        return self.cProxy.character_bio_parser.parse(cont,
+                                                    getRefs=self._getRefs)
+
+    def get_character_episodes(self, characterID):
+        cont = self._retrieve(imdbURL_character_main % characterID +
+                                'filmoseries')
+        return self.cProxy.character_series_parser.parse(cont)
+
+    def get_character_quotes(self, characterID):
+        cont = self._retrieve(imdbURL_character_main % characterID + 'quotes')
+        return self.cProxy.character_quotes_parser.parse(cont,
+                                                    getRefs=self._getRefs)
+
+    def _search_company(self, name, results):
+        cont = self._get_search_content('co', name, results)
+        url = self.urlOpener._last_url
+        return self.scompProxy.search_company_parser.parse(cont, url=url,
+                                                    results=results)['data']
+
+    def get_company_main(self, companyID):
+        cont = self._retrieve(imdbURL_company_main % companyID)
+        ret = self.compProxy.company_main_parser.parse(cont)
+        return ret
 
 
