@@ -8,7 +8,8 @@ E.g., for when searching for the title "the passion", the parsed
 page would be:
     http://akas.imdb.com/find?q=the+passion&tt=on&mx=20
 
-Copyright 2004-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
+               2008 H. Turgut Uyar <uyar@tekir.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,8 +26,11 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
-from imdb.utils import analyze_title, analyze_name
-from utils import ParserBase
+import re
+from imdb import imdbURL_base
+from imdb.utils import analyze_title, analyze_name, analyze_company_name
+from utils import ParserBase, DOMParserBase, Attribute, Extractor, \
+        analyze_imdbid
 from imdb.Movie import Movie
 
 
@@ -109,7 +113,7 @@ class BasicMovieParser(ParserBase):
             if st and self.__seriesID:
                 d_title = analyze_title(st, canonical=1)
                 m = Movie(movieID=str(self.__seriesID), data=d_title,
-                            accessSystem='http')
+                            accessSystem=self._as, modFunct=self._modFunct)
                 self._result['kind'] = u'episode'
                 self._result['episode of'] = m
             self._series_title = u''
@@ -149,10 +153,20 @@ class BasicMovieParser(ParserBase):
                 self._in_series_info = 1
 
 
+def _dontChange(s, *args, **kwds):
+    """Return the name (useful for characters objects)."""
+    return {'name': s}
+
+def _analyze_company_name(n, *args, **kwds):
+    """analyze_company_name doesn't accept the 'canonical' paramter."""
+    return analyze_company_name(n, stripNotes=True)
+
+
 class HTMLSearchMovieParser(ParserBase):
     """Parse the html page that the IMDb web server shows when the
     "new search system" is used, for both movies and persons."""
-    # Customizations for movie and person parsers.
+    # Customizations for movie, person and character parsers.
+
     _k = {
         'movie':
             {'analyze_f': analyze_title,
@@ -163,6 +177,16 @@ class HTMLSearchMovieParser(ParserBase):
             {'analyze_f': analyze_name,
             'link': '/name',
             'in title': 'imdb name'},
+
+        'character':
+            {'analyze_f': _dontChange,
+            'link': '/character',
+            'in title': 'imdb search'},
+
+        'company':
+            {'analyze_f': _analyze_company_name,
+            'link': '/company',
+            'in title': 'imdb company search'}
     }
 
     def _init(self):
@@ -215,11 +239,27 @@ class HTMLSearchMovieParser(ParserBase):
         self._is_title = False
         self._current_imdbID = None
 
+    def do_img(self, attrs):
+        # Skips mini-posters in the results (they are there, if
+        # we don't use the IMDbPYweb's account, performing the search).
+        if not self._in_table: return
+        #if self.kind == 'character': return
+        #src = self.get_attr_value(attrs, 'src')
+        #if src and not src.lower().endswith('/b.gif'):
+        #    if self._col_nr == 1:
+        #        self._col_nr = 0
+
     def end_td(self):
         if self._in_table and self._is_title and self._current_imdbID and \
                 self._col_nr == 3:
             # We should have got the title/name.
             title = self._current_ton.strip()
+            if not title:
+                self._current_ton = u''
+                self._current_imdbID = u''
+                self._is_title = False
+                self._no_more = 0
+                return
             tup = (self._current_imdbID,
                     self._k[self.kind]['analyze_f'](title, canonical=1))
             self._results.append(tup)
@@ -252,7 +292,9 @@ class HTMLSearchMovieParser(ParserBase):
     def end_small(self): pass
 
     def do_br(self, attrs):
-        self._no_more = True
+        if not self.kind == 'character':
+            if self._col_nr > 3:
+                self._no_more = True
 
     def _handle_data(self, data):
         if self._stop:
@@ -263,7 +305,7 @@ class HTMLSearchMovieParser(ParserBase):
         if self._in_table and self._col_nr == 3 and not self._no_more:
             self._current_ton += data
         elif self._reading_page_title:
-            dls = data.strip().lower()
+            dls = data.strip().lower().replace('  ', ' ')
             if not dls.startswith(self._k[self.kind]['in title']):
                 # XXX: a direct result!
                 #      Interrupt parsing, and retrieve data using a
@@ -278,7 +320,91 @@ class HTMLSearchMovieParser(ParserBase):
                 self._results = bmp.parse(rawdata)['data']
 
 
-# The used object.
-search_movie_parser = HTMLSearchMovieParser()
+class DOMBasicMovieParser(DOMParserBase):
+    """Simply get the title of a movie and the imdbID.
+
+    It's used by the DOMHTMLSearchMovieParser class to return a result
+    for a direct match (when a search on IMDb results in a single
+    movie, the web server sends directly the movie page."""
+    # Stay generic enough to be used also for other DOMBasic*Parser classes.
+    _titleAttrPath = ".//text()"
+    _linkPath = "//a[starts-with(@href, '/title/tt')]"
+    def _init(self):
+        self.extractors = [Extractor(label='title',
+                                path="//h1",
+                                attrs=Attribute(key='title',
+                                                path=self._titleAttrPath)),
+                            Extractor(label='link',
+                                path=self._linkPath,
+                                attrs=Attribute(key='link', path="./@href"))]
+
+    # Remove 'More at IMDb Pro' links.
+    preprocessors = [(re.compile(r'<span class="pro-link".*?</span>'), '')]
 
 
+class DOMHTMLSearchMovieParser(DOMParserBase):
+    """Parse the html page that the IMDb web server shows when the
+    "new search system" is used, for movies."""
+
+    _BaseParser = DOMBasicMovieParser
+    _notDirectHitTitle = '<title>imdb title'
+
+    _attrs = [Attribute(key='data',
+                        multi=True,
+                        path={
+                            'link': "./a[1]/@href",
+                            'info': ".//text()"
+                            },
+                        postprocess=lambda x: (
+                            analyze_imdbid(x.get('link') or u''),
+                            analyze_title(x.get('info') or u'', canonical=1)
+                        ))]
+    extractors = [Extractor(label='search',
+                        path="//td[3]/a[starts-with(@href, '/title/tt')]/..",
+                        attrs=_attrs)]
+    def _init(self):
+        self.url = u''
+
+    def _reset(self):
+        self.url = u''
+
+    def preprocess_string(self, html_string):
+        if self._notDirectHitTitle in html_string[:1024].lower():
+            return html_string
+        dbme = self._BaseParser(useModule=self._useModule)
+        res = dbme.parse(html_string, url=self.url)
+        if not res: return u''
+        res = res['data']
+        if not res: return u''
+        if not 'title' in res: return u''
+        if 'link' in res:
+            link = res['link']
+        else:
+            # Tries to cope with companies for which links to pro.imdb.com
+            # are missing.
+            link = self.url.replace(imdbURL_base[:-1], '')
+        title = res['title']
+        if not (link and title): return u''
+        link = link.replace('http://pro.imdb.com', '')
+        new_html = '<td></td><td></td><td><a href="%s">%s</a></td>' % (link,
+                                                                    title)
+        return new_html
+
+    def postprocess_data(self, data):
+        if not data.has_key('data'):
+            data['data'] = []
+        results = getattr(self, 'results', None)
+        if results is not None:
+            data['data'][:] = data['data'][:results]
+        return data
+
+    def add_refs(self, data):
+        return data
+
+
+
+_OBJECTS = {
+        'search_movie_parser': ((DOMHTMLSearchMovieParser,
+                                HTMLSearchMovieParser), None)
+}
+
