@@ -6,7 +6,7 @@ IMDb's data for mobile systems.
 the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "mobile".
 
-Copyright 2005-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2005-2008 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -192,8 +192,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             if not (mid and title): return res
             res[:] = [(str(mid[0]), analyze_title(title, canonical=1))]
         else:
-            lis = _findBetween(cont, 'td valign="top">',
-                                ['</td>', '<br>', '<br/>'])
+            lis = _findBetween(cont, 'td valign="top">', ['</td>', '<small>'])
             for li in lis:
                 imdbid = re_imdbID.findall(li)
                 mtitle = _unHtml(li)
@@ -296,7 +295,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             if cvurl: d['cover url'] = cvurl[0]
         genres = _findBetween(cont, 'href="/Sections/Genres/', '/')
         if genres: d['genres'] = genres
-        ur = _findBetween(cont, 'User Rating:</b>', ' votes</a>)', maxRes=1)
+        ur = _findBetween(cont, '<div class="meta">', '</div>', maxRes=1)
         if ur:
             rat = _findBetween(ur[0], '<b>', '</b>', maxRes=1)
             if rat:
@@ -308,10 +307,11 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                         d['rating'] = rat
                     except ValueError:
                         pass
-            vi = ur[0].rfind('(')
-            if vi != -1 and ur[0][vi:].find('await') == -1:
+            vi = ur[0].rfind('tn15more">')
+            if vi != -1 and ur[0][vi+10:].find('await') == -1:
                 try:
-                    votes = int(_unHtml(ur[0][vi+1:]).strip().replace(',', ''))
+                    votes = _unHtml(ur[0][vi+10:]).replace('votes', '').strip()
+                    votes = int(votes.replace(',', ''))
                     d['votes'] = votes
                 except ValueError:
                     pass
@@ -355,8 +355,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if akas:
             # For some reason, here <br> is still used in place of <br/>.
             akas[:] = [x for x in akas[0].split('<br>') if x.strip()]
-            akas = [_unHtml(x).replace(' (','::(', 1).replace(' [','::[')
-                    for x in akas]
+            akas = [_unHtml(x).replace(' (','::(', 1) for x in akas]
             if 'more' in akas: akas.remove('more')
             d['akas'] = akas
         mpaa = _findBetween(cont, 'MPAA</a>:', '</div>', maxRes=1)
@@ -365,7 +364,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if runtimes:
             runtimes = runtimes[0]
             runtimes = [x.strip().replace(' min', '').replace(' (', '::(', 1)
-                    for x in runtimes.split('/')]
+                    for x in runtimes.split('|')]
             d['runtimes'] = runtimes
         if kind == 'episode':
             # number of episodes.
@@ -379,37 +378,38 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                     d['number of episodes'] = epsn
         country = _findBetween(cont, 'Country:</h5>', '</div>', maxRes=1)
         if country:
-            country[:] = country[0].split(' / ')
+            country[:] = country[0].split(' | ')
             country[:] = ['<a %s' % x for x in country if x]
             country[:] = [_unHtml(x.replace(' <i>', '::')) for x in country]
             if country: d['countries'] = country
         lang = _findBetween(cont, 'Language:</h5>', '</div>', maxRes=1)
         if lang:
-            lang[:] = lang[0].split(' / ')
+            lang[:] = lang[0].split(' | ')
             lang[:] = ['<a %s' % x for x in lang if x]
             lang[:] = [_unHtml(x.replace(' <i>', '::')) for x in lang]
             if lang: d['languages'] = lang
         col = _findBetween(cont, '"/List?color-info=', '</div>')
         if col:
-            col[:] = col[0].split(' / ')
+            col[:] = col[0].split(' | ')
             col[:] = ['<a %s' % x for x in col if x]
             col[:] = [_unHtml(x.replace(' <i>', '::')) for x in col]
             if col: d['color info'] = col
         sm = _findBetween(cont, '/List?sound-mix=', '</div>', maxRes=1)
         if sm:
-            sm[:] = sm[0].split(' / ')
+            sm[:] = sm[0].split(' | ')
             sm[:] = ['<a %s' % x for x in sm if x]
             sm[:] = [_unHtml(x.replace(' <i>', '::')) for x in sm]
             if sm: d['sound mix'] = sm
         cert = _findBetween(cont, 'Certification:</h5>', '</div>', maxRes=1)
         if cert:
-            cert[:] = cert[0].split(' / ')
+            cert[:] = cert[0].split(' | ')
             cert[:] = [_unHtml(x.replace(' <i>', '::')) for x in cert]
             if cert: d['certificates'] = cert
-        plotoutline = _findBetween(cont, 'Plot Outline:</h5>', ['<a ','</div>'],
+        plotoutline = _findBetween(cont, 'Plot:</h5>', ['<a ','</div>'],
                                     maxRes=1)
         if plotoutline:
             plotoutline = plotoutline[0].strip()
+            plotoutline = plotoutline.rstrip('|').rstrip()
             if plotoutline: d['plot outline'] = plotoutline
         return {'data': d}
 
@@ -458,7 +458,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             res[:] = [(str(pid[0]), analyze_name(name, canonical=1))]
         else:
             lis = _findBetween(cont, 'td valign="top">',
-                                ['<small', '</td>', '<br'])
+                                ['<small', '</td>', '<br> aka'])
             for li in lis:
                 pid = re_imdbID.findall(li)
                 pname = _unHtml(li)
@@ -496,7 +496,11 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         akas = _findBetween(s, 'Alternate Names:</h5>', ('</div>',
                             '<br/><br/>'), maxRes=1)
         if akas:
-            akas = _unHtml(akas[0]).split(' / ')
+            akas = akas[0]
+            if akas.find(' | ') != -1:
+                akas = _unHtml(akas).split(' | ')
+            else:
+                akas = _unHtml(akas).split(' / ')
             if akas: r['akas'] = akas
         hs = _findBetween(s, 'name="headshot"', '</a>', maxRes=1)
         if hs:
@@ -610,7 +614,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         misc_sects[:] = [x for x in misc_sects if len(x) == 2]
         for sect, data in misc_sects:
             sect = sect.lower().replace(':', '').strip()
-            if d.has_key(sect): continue
+            if d.has_key(sect) and sect != 'mini biography': continue
             if sect == 'salary': sect = 'salary history'
             elif sect == 'spouse': continue
             elif sect == 'nickname': sect = 'nick names'
@@ -648,6 +652,9 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                                             bio[:byidx].rstrip())
                     ndata.append(bio)
                 data[:] = ndata
+                if 'mini biography' in d:
+                    d['mini biography'].append(ndata[0])
+                    continue
             d[sect] = data
         return {'data': d}
 
@@ -671,9 +678,10 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             sects = _findBetween(cont, '<b>Popular Characters</b>', '</table>')
             sects += _findBetween(cont, '<b>Characters', '</table>')
             for sect in sects:
-                lis = _findBetween(sect, 'td valign="top">',
+                lis = _findBetween(sect, '<a href="/character/',
                                     ['<small', '</td>', '<br'])
                 for li in lis:
+                    li = '<%s' % li
                     pid = re_imdbID.findall(li)
                     pname = _unHtml(li)
                     if not (pid and pname): continue
@@ -701,6 +709,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 bio = bio.replace('</h4>', '::')
                 bio = bio.replace('\n', ' ')
                 bio = bio.replace('<br>', '\n')
+                bio = bio.replace('<br/>', '\n')
                 bio = subSGMLRefs(re_unhtmlsub('', bio).strip())
                 bio = bio.replace(' ::', '::').replace(':: ', '::')
                 if bio:
